/**
 * @file
 * Provides blazy PhotoSwipe loader.
 *
 * Provides own PhotoSwipe loader for advanced contents with videos, etc.
 */

(function ($, Drupal, drupalSettings, PhotoSwipe, PhotoSwipeUI_Default) {

  'use strict';

  /**
   * Blazy Photoswipe utility functions.
   *
   * @param {int} i
   *   The index of the current element.
   * @param {HTMLElement} elm
   *   The gallery HTML element.
   */
  function blazyPhotoSwipe(i, elm) {
    var $elm = $(elm);
    var opts = drupalSettings.photoswipe.options || {};
    var pswpElm = $('.pswp')[0];

    /**
     * Triggers when user clicks on thumbnail.
     *
     * @param {jQuery.Event} event
     *   The event triggered by a `click` event.
     */
    function openBox(event) {
      event.preventDefault();

      // With a mix of (non-)lightboxed contents: image, video, Facebook,
      // Instagram, etc., some may not always be lightboxed, so filter em out.
      var $target = $(event.currentTarget);
      var parents = '.box, .field__item, .grid, .slide, .views-row';
      var $item = $target.closest(elm).find('[data-photoswipe-trigger]').closest(parents);
      var $index = $target.closest(parents);
      var index = $index.length ? $item.index($index) : 0;

      box(index);
    }

    /**
     * Open the PhotoSwipe box.
     *
     * Code taken from http://photoswipe.com/documentation/getting-started.html
     *
     * @param {int} delta
     *   The start index of the slide, not the current slide index.
     */
    function box(delta) {
      var items = [];
      delta = delta > 0 ? delta : 0;

      $elm.find('[data-photoswipe-trigger]').each(function (i) {
        var that = this;
        var $link = $(that);
        var $caption = $link.next('.litebox-caption');
        var media = $link.data('media') || {};
        var url = $link.attr('href');
        var tnUrl = $('[data-thumb]', that).data('thumb');
        var item = {};

        // Save link to element for getThumbBoundsFn.
        item.el = $link[0];
        item.w = media.width;
        item.h = media.height;
        item.i = i;
        item.type = media.type;

        if ($caption.length) {
          item.title = $caption.html();
        }

        if (tnUrl) {
          item.msrc = tnUrl;
        }

        // @todo: Supports audio, inline, etc. if time permits.
        if (media.type === 'image') {
          item.src = url;
        }
        else {
          item.html = Drupal.theme('blazyPhotoSwipeMedia', {item: item});
        }

        items.push(item);
      });

      // Define options.
      opts.index = delta;

      // Prevent HTML or Video cover from triggering close.
      opts.clickToCloseNonZoomable = false;

      // Define gallery index (for URL).
      opts.galleryUID = $elm.data('pswp-uid');

      opts.getThumbBoundsFn = function (i) {
        var pageYScroll = window.pageYOffset || document.documentElement.scrollTop;
        var rect = $(items[i].el)[0].getBoundingClientRect();

        return {x: rect.left, y: rect.top + pageYScroll, w: rect.width};
      };

      // Pass data to PhotoSwipe and initialize it.
      var pswp = new PhotoSwipe(pswpElm, PhotoSwipeUI_Default, items, opts);

      // Listen to events.
      pswp.listen('beforeChange', function () {
        Drupal.attachBehaviors(pswpElm);
      });

      pswp.listen('afterChange', function () {
        closeVideo();
      });

      pswp.listen('pointerDown', function () {
        $(pswpElm).addClass('pswp--dragging');
      });

      pswp.listen('pointerUp', function () {
        $(pswpElm).removeClass('pswp--dragging');
      });

      pswp.init();
    }

    /**
     * Parse picture index and gallery index from URL (#&pid=1&gid=2)
     *
     * Code taken from http://photoswipe.com/documentation/getting-started.html
     * and adjusted accordingly.
     *
     * @return {array}
     *   Return array of URL params if available, else empty array.
     */
    function parseHash() {
      var hash = window.location.hash.substring(1);
      var params = {};

      if (hash.length < 5) {
        return params;
      }

      var vars = hash.split('&');
      for (var i = 0; i < vars.length; i++) {
        if (!vars[i]) {
          continue;
        }
        var pair = vars[i].split('=');
        if (pair.length < 2) {
          continue;
        }
        params[pair[0]] = pair[1];
      }

      if (!params.hasOwnProperty('pid')) {
        return params;
      }
      params.pid = parseInt(params.pid, 10);

      return params;
    }

    /**
     * Triggers when user clicks on buttons, or after changing slides.
     */
    function closeVideo() {
      if ($('.is-playing', pswpElm).length) {
        $('.is-playing .media__icon--close', pswpElm).trigger('click');
      }
    }

    // Build PhotoSwipe gallery.
    $elm.attr('data-pswp-uid', (i + 1));
    $elm.on('click.pswp', '[data-photoswipe-trigger]', openBox);
    $(pswpElm).on('click.mediaClose', '.pswp__button', closeVideo);

    // Parse URL, and auto-open gallery if it contains #&pid=3&gid=1 params.
    var hashData = parseHash();
    if (hashData.pid > 0) {
      box(hashData.pid - 1);
    }
  }

  /**
   * Theme function for a blazy photoswipe video.
   *
   * @param {object} settings
   *   An object with the following keys:
   * @param {array} settings.item
   *   The array of item properties containing: el, w, h.
   *
   * @return {HTMLElement}
   *   Returns a HTMLElement object.
   */
  Drupal.theme.blazyPhotoSwipeMedia = function (settings) {
    var item = settings.item;
    var $link = $(item.el);
    var alt = $('img', item.el).attr('alt');
    var pad = Math.round(((item.h / item.w) * 100), 2);
    var boxUrl = $link.data('boxUrl');
    var embedUrl = $link.attr('href');
    var player = item.type === 'video' ? ' media--player' : '';
    var html;

    html = '<div class="pswp__item--html media-wrapper" style="width:' + item.w + 'px">';
    html += '<div class="media media--pswp media--switch' + player + ' media--ratio media--ratio--fluid" style="padding-bottom: ' + pad + '%">';

    if (item.type === 'video') {
      html += '<img src="' + boxUrl + '" class="media__image media__element pswp__img" alt="' + Drupal.t(alt) + '"/>';
      html += '<span class="media__icon media__icon--close"></span>';
      html += '<span class="media__icon media__icon--play" data-url="' + embedUrl + '"></span>';
    }

    html += '<iframe src="about:blank" data-src="' + embedUrl + '" width="' + item.w + '" height="' + item.h + '" class="b-lazy media__iframe media__element" allowfullscreen></iframe>';
    html += '</div></div>';

    return html;
  };

  /**
   * Attaches PhotoSwipe behavior to HTML element [data-photoswipe-gallery].
   *
   * @type {Drupal~behavior}
   */
  Drupal.behaviors.blazyPhotoSwipe = {
    attach: function (context) {
      // If body has no container for photoswipe gallery, append it.
      if ('container' in drupalSettings.photoswipe && !$('.pswp').length) {
        $('body').append(drupalSettings.photoswipe.container);
      }

      $('[data-photoswipe-gallery]', context).once('blazyPhotoSwipe').each(blazyPhotoSwipe);
    }
  };

}(jQuery, Drupal, drupalSettings, PhotoSwipe, PhotoSwipeUI_Default));
